<?php
/**
 * @package   Yandex pay
 * @author    Outcode https://outcode.ru/
 * @license   http://www.gnu.org/licenses/gpl-2.0.html GNU/GPLv2 only
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

if(!class_exists( 'VmConfig' )){
  require(JPATH_ROOT .'/administrator/components/com_virtuemart/helpers/config.php');
}

class YandexpayControllerWrapper extends JControllerLegacy{

  function __construct($config = array()){
    JLoader::register('YandexPayAPI', JPATH_ROOT . '/libraries/yandexapi/yandexapi.php');
    JLoader::register('YPHelpers', JPATH_ROOT . '/libraries/yandexapi/YPHelpers.php');

    VmConfig::loadConfig();
    $this->productModel = VmModel::getModel ('product');

    $this->YPHelpers = new YPHelpers(); 
    $this->Settings = $this->YPHelpers->getSettings();
    $this->YandexPayAPI = new YandexPayAPI($this->Settings);  
    $this->App = JFactory::getApplication();
    $this->Document = JFactory::getDocument();

    parent::__construct( $config );
  }

  function display($cachable = false, $urlparams = false){  

    $app = JFactory::getApplication();
    $api = $app->input->get('api', '', 'string');
    $content = trim(file_get_contents('php://input'));
    $order_id = $this->input->get('orderId', '', 'STRING');
    $orderRefund = $this->input->get('orderRefund', '', 'STRING');

    switch ($api) {
      case 'v1/order/render':        
        $this->wrapperYandexOrderRender($content);
        break;
      case 'v1/order/create':
        $this->wrapperYandexOrderCreate($content);
        break;
      case 'v1/webhook':
        $this->wrapperYandexWebhook($content);
        break;
      case 'v1/clearcart':
        $this->clearCart();
        break;
      case 'v1/OrderDetailsForRefund':
        $this->orderDetailsForRefund($order_id);
        break;
      case 'v1/OrderRefund':
        $this->orderRefund($orderRefund);
        break;
      case 'v1/pickup-options':
        
        break;
      case 'v1/pickup-option-details':
        
        break;
      case 'v1/yandex-delivery/adjust-options ':
        
        break;
      default;
        JError::raiseError( 404, JText::_('COM_YANDEXPAY_PAGE_NOT_FOUND') );
        break;
    }  

    jexit();    
  }
  
  /*
   *  Запрос информации для отображения корзины
   *  /v1/order/render
   *
   *  $token - после валидации содержит информацию о заказе
   *  
  */
  function wrapperYandexOrderRender($content){
    $token = $this->YandexPayAPI->JWTValidation($content);

    if(isset($token->api_message)){ // Валидация токена не успешна  
      $this->Document->setMimeEncoding( 'application/json' );
      JResponse::setHeader( 'Status', $token->api_message->reason );
      JResponse::sendHeaders();
      echo json_encode( $token->api_message );
      jexit();
    }
    
    if(!isset($token->cart)){
      $messages = $this->YandexPayAPI->errorMessages(403, 'There is no basket in tokenet');
      $this->Document->setMimeEncoding( 'application/json' );
      JResponse::setHeader( 'Status', $messages->api_message->reason );
      JResponse::sendHeaders();
      echo json_encode( $messages->api_message );
      jexit();
    }
    
    /*
     * Настройки магазина
     *
    */
    $store_settings = [
      'currencyCode' => $token->currencyCode
    ];
    
    /*
     * Информация о доставке
     *
    */  
    $shipping = null;
    
    /*
     * Информация о товарах в заказе
     *
    */
    $cart = ['cart_total' => 0];

    foreach($token->cart->items as $item){
      $product = $this->productModel->getProduct($item->productId); 
      $virtuemart_product_id = $product->virtuemart_product_id;  

      if($item->productId == $virtuemart_product_id){
        $qty = $item->quantity->count; 
        $price = $product->prices['salesPrice'];
        $sell_price = $product->prices['salesPrice'];

        $cart['items'][] = [
          'productId' => $item->productId,
          'unitPrice' => number_format($price, 2, '.', ''),
          'discountedUnitPrice' => number_format($sell_price, 2, '.', ''),
          'subtotal' => number_format($price * $qty, 2, '.', ''),
          'total' => number_format($sell_price * $qty, 2, '.', ''),
          'title' => $product->product_name,
          'qty_count' => number_format($qty, 2, '.', ''),       
        ];
        
        $cart['cart_total'] += $sell_price * $qty;    
      }  
    }    
  
    // Проверяем наличие промокода из формы оплаты
    $coupon = $this->YPHelpers->checkCoupon($token);
    $coupons = null;
    
    if($token->metadata == 'product_card'){
      $coupons = $this->YPHelpers->getCouponAmount($token->orderId, $token->metadata, $coupon, $token);
    }
    
    if($token->metadata == 'cart'){
      $coupons = $this->YPHelpers->getCouponAmount($token->orderId, $token->metadata, $coupon);
    }
    
    if($token->metadata == 'native'){
      $coupons = $this->YPHelpers->getCouponAmount($token->orderId, $token->metadata, $coupon);
    }   

    if(!is_null($coupons) && $coupons != 'not_valid'){
      $cart['cart_total'] = $cart['cart_total'] - $coupons['amount'];
      $cart['coupons'] = [
        'value' => $coupons['code'],
        'status' => 'VALID',
        'description' => JText::_("COM_YANDEXPAY_TXT_42").': '.$coupons['code']
      ];
      
      $cart['discounts'] = [
        'discountId' => $coupons['code'],
        'amount' => number_format($coupons['amount'], 2, '.', ''),
        'description' => JText::_("COM_YANDEXPAY_TXT_42").': '.$coupons['code']
      ];
    }

    $cart['cart_total'] = number_format($cart['cart_total'], 2, '.', '');
    $output = $this->YandexPayAPI->yandexOrderRender($store_settings, $cart, $shipping);
    
    $this->Document->setMimeEncoding( 'application/json' );
    JResponse::setHeader( 'Content-Type', 'application/json' );
    JResponse::sendHeaders();
    echo json_encode( $output );
    jexit(); 
  }
 
  /*
   *  Создание заказа
   *  /v1/order/create
   *
   *  $token - после валидации содержит информацию о заказе
   *  
  */
  function wrapperYandexOrderCreate($content){
    $token = $this->YandexPayAPI->JWTValidation($content);

    if(isset($token->api_message)){ // Валидация токена не успешна  
      $this->Document->setMimeEncoding( 'application/json' );
      JResponse::setHeader( 'Status', $token->api_message->reason );
      JResponse::sendHeaders();
      echo json_encode( $token->api_message );
      jexit();
    }

    $items            = $token->cart->items;  
    $shipping         = $token->shippingAddress;
    $shippingContact  = $token->shippingContact;

    $thoroughfare = $shipping->zip.', '.$shipping->country.', '.$shipping->region.', ';
    $thoroughfare .= $shipping->locality.', '.$shipping->street.' '.$shipping->building.', ';
    $thoroughfare .= JText::_("COM_YANDEXPAY_TXT_32").'';
    $thoroughfare .= $shipping->room;
    
    $user = [
      'last_name' => $shippingContact->lastName,
      'first_name' => $shippingContact->firstName,
      'phone_1' => $shippingContact->phone,
      'address_1' => $thoroughfare,
      'city' => $shipping->locality,
      'zip' => $shipping->zip,
      'email' => $shippingContact->email,
    ]; 

    $ids = $qts = [];

    foreach($items as $item){    
      $ids[] = $item->productId;
      $qts[] = $item->quantity->count;
    }

    $shipping_amount = $token->shippingMethod->courierOption->amount;
    $comment = $shipping->comment;
    
    // Проверяем наличие промокода из формы оплаты
    $coupon = $this->YPHelpers->checkCoupon($token);

    // Заказ из карточки товара
    if($token->metadata == 'product_card'){
      $discount = $this->YPHelpers->getCouponAmount($token->orderId, $token->metadata, $coupon, $token);
      $order_id = $this->YPHelpers->createOrder($ids, $qts, $user, $shipping_amount, $comment, $discount);
    }

    // Заказ из корзины
    if($token->metadata == 'cart'){
      //$discount = $this->YPHelpers->getCouponAmount($token->orderId);
      $discount = $this->YPHelpers->getCouponAmount($token->orderId, $token->metadata, $coupon);
      $order_id = $this->YPHelpers->createOrder($ids, $qts, $user, $shipping_amount, $comment, $discount);
    }

    // Заказ нативным способом оплаты
    if($token->metadata == 'native'){
      $order_id = explode('_', $token->orderId);
      $order_id = $order_id[0];
      $discount = $this->YPHelpers->getCouponAmount($token->orderId, $token->metadata, $coupon);
      $this->YPHelpers->updateNativeOrder($order_id, $shipping_amount, $comment, $discount);
    }

    $return = [
      'order_id' => $order_id.'_'.time(),
      'receipt' => null,
    ];
    
    $this->YPHelpers->saveYPOrderId($order_id, $return['order_id']);

    $output = $this->YandexPayAPI->yandexOrderCreate($return);    

    $this->Document->setMimeEncoding( 'application/json' );
    JResponse::setHeader( 'Content-Type', 'application/json' );
    JResponse::sendHeaders();
    echo json_encode( $output );
    jexit(); 
  } 
   
  /*
   *  Изменении статуса заказа
   *  /v1/webhook
   *
  */
  function wrapperYandexWebhook($content){  
    $token = $this->YandexPayAPI->JWTValidation($content);

    if(isset($token->api_message)){ // Валидация токена не успешна  
      $this->Document->setMimeEncoding( 'application/json' );
      JResponse::setHeader( 'Status', $token->api_message->reason );
      JResponse::sendHeaders();
      echo json_encode( $token->api_message );
      jexit();
    }
    
    $original_order_id = $token->order->orderId;
    $order_id = explode('_', $token->order->orderId);
    $token->order->orderId = $order_id[0];
    
    $order_total = $this->YPHelpers->getOrderTotal($token->order->orderId);

    $data = [
      'error' => '',
      'token' => $token,
    ];
    
    if(is_null($order_total)){      
      $data['error'] = 'ORDER_NOT_FOUND';
      $output = $this->YandexPayAPI->yandexOrderWebhook($data);
      $this->Document->setMimeEncoding( 'application/json' );
      JResponse::setHeader( 'Content-Type', 'application/json' );
      JResponse::sendHeaders();
      echo json_encode( $output );
      jexit(); 
    }

    // Изменение статуса платежа
    if($token->event == 'ORDER_STATUS_UPDATED'){

      switch ($token->order->paymentStatus) {
        case 'PENDING':
          $message = $token->order->paymentStatus.': '.JText::_("COM_YANDEXPAY_TXT_33");        
          $this->YPHelpers->updateOrderPaidStatus($token->order->orderId, 0, $token->eventTime);
          $this->YPHelpers->updateOrderComment($token->order->orderId, $message); 
          break;
        case 'AUTHORIZED':
          $message = $token->order->paymentStatus.': '.JText::_("COM_YANDEXPAY_TXT_34");        
          $this->YPHelpers->updateOrderPaidStatus($token->order->orderId, 0, $token->eventTime);
          $this->YPHelpers->updateOrderComment($token->order->orderId, $message);
          $this->wrapperYandexDeliveryordersCapture($original_order_id);
          break;
        case 'CAPTURED':   // Оплачено 
          $message = $token->order->paymentStatus.': '.JText::_("COM_YANDEXPAY_TXT_35");        
          $this->YPHelpers->updateOrderPaidStatus($token->order->orderId, $order_total, $token->eventTime);
          $this->YPHelpers->updateOrderComment($token->order->orderId, $message);
          break;
        case 'VOIDED':
          $message = $token->order->paymentStatus.': '.JText::_("COM_YANDEXPAY_TXT_36");        
          $this->YPHelpers->updateOrderPaidStatus($token->order->orderId, 0, $token->eventTime);
          $this->YPHelpers->updateOrderComment($token->order->orderId, $message);
          break;
        case 'REFUNDED': // Совершён возврат средств за заказ
          $message = $token->order->paymentStatus.': '.JText::_("COM_YANDEXPAY_TXT_37");        
          $this->YPHelpers->updateOrderPaidStatus($token->order->orderId, 0, $token->eventTime);
          $this->YPHelpers->updateOrderComment($token->order->orderId, $message);
          $this->YPHelpers->updateOrderStatus($token->order->orderId, 'R');
          break;
        case 'PARTIALLY_REFUNDED': // Совершён частичный возврат средств за заказ
          $message = $token->order->paymentStatus.': '.JText::_("COM_YANDEXPAY_TXT_38");        
          $this->YPHelpers->updateOrderPaidStatus($token->order->orderId, 0, $token->eventTime);
          $this->YPHelpers->updateOrderComment($token->order->orderId, $message);
          $this->YPHelpers->updateOrderStatus($token->order->orderId, 'R');
          break;
        case 'FAILED':
          $message = $token->order->paymentStatus.': '.JText::_("COM_YANDEXPAY_TXT_39");        
          $this->YPHelpers->updateOrderPaidStatus($token->order->orderId, 0, $token->eventTime);
          $this->YPHelpers->updateOrderComment($token->order->orderId, $message);
          break;       
      } 
    }

    $output = $this->YandexPayAPI->yandexOrderWebhook($data);

    $this->Document->setMimeEncoding( 'application/json' );
    JResponse::setHeader( 'Content-Type', 'application/json' );
    JResponse::sendHeaders();
    echo json_encode( $output );
    jexit(); 
  }
  
  /*
   *  Подтверждение платежа
   *
   */
  function wrapperYandexDeliveryordersCapture($order_id){
    $attempt = 0;
    $order_total = $this->YPHelpers->getOrderTotal($order_id);

    if(is_null($order_total)){
      JLog::add('(# '.$order_id.') '.JText::_("COM_YANDEXPAY_TXT_40"), JLog::ERROR, 'YandexPay');
      return false;
    }

    while ($attempt <= 5) {   
      $attempt++;
      $result = $this->YandexPayAPI->yandexDeliveryordersCapture($order_id, $attempt);

      if($result->code == 200){
        break;
      }
      
      if($result->code == 400){
        $data = json_decode($result->data);
        $message = '(# '.$order_id.') ';
        $message .= JText::_("COM_YANDEXPAY_TXT_41").' ';
        $message .= 'status: '.$data->status.', ';
        $message .= 'reasonCode: '.$data->reasonCode.', ';
        $message .= 'reason: '.$data->reason;
        $this->YPHelpers->updateOrderComment($order_id, $message);   

        return false;
      }
      
      if($result->code == 404){    
        $message = '(# '.$order_id.') ';
        $message .= JText::_("COM_YANDEXPAY_TXT_40");
        $this->YPHelpers->updateOrderComment($order_id, $message);

        return false;
      }    
    }
    
    return true;
  } 
  
  /*
   *  Очищаем корзину клиента
   *  /v1/clear-cart
   *
   */
  function clearCart(){  
    $cart = VirtueMartCart::getCart();
		//$cart->emptyCart(); 
		$cart->removeCartFromSession(); 
    
    echo json_encode( ['message' => 'clear-cart'] );
    jexit();  
  }
  
  function orderDetailsForRefund($order_id){ 
    $order = $this->YandexPayAPI->yandexGetOrderInfo($order_id);
    echo json_encode( ['order' => $order->data->order] );
    jexit();  
  }
  
  function orderRefund($orderRefund){
    $refund = json_decode($orderRefund);
  
    $order = $this->YandexPayAPI->yandexGetOrderInfo($refund->orderId);    
    $result = $this->YandexPayAPI->yandexRefund($order->data->order, $refund);

    echo json_encode( ['result' => $result] );
    jexit(); 
  }
 
}